//
// Created by robin on 2/1/18.
//

#include "QDnsTcpServerClient.h"

namespace QDns
{
    namespace Server
    {
        QDnsTcpServerClient::QDnsTcpServerClient(QTcpSocket *socket, QObject* parent)
            : QObject(parent)
            , m_socket(socket)
            , m_size(0)
        {
            m_socket->setParent(this);
            connect(m_socket, &QTcpSocket::readyRead, this, &QDnsTcpServerClient::onReadyRead);
            // TODO implement timeout (~2 mins, RFC 7766)
        }

        void QDnsTcpServerClient::sendAnswer(const QDns::Server::QDnsServer::SendAnswerParams &answerParams)
        {
            auto answerRaw = answerParams.answer.serialize(); // TODO split packet if too big (>65535)
            QByteArray size;
            QDataStream dataStream(&size, QIODevice::WriteOnly);
            dataStream << (quint16) answerRaw.size();
            m_socket->write(size.constData(), size.size());
            m_socket->write(answerRaw.constData(), answerRaw.size());
        }

        void QDnsTcpServerClient::onReadyRead()
        {
            m_buffer.append(m_socket->readAll());

            bool readAgain;

            do
            {
                readAgain = false;
                if (m_buffer.size() > 2 && m_size == 0)
                {
                    QDataStream dataStream(m_buffer);
                    dataStream >> m_size;
                }
                if (m_size != 0 && m_buffer.size() >= m_size + 2)
                {
                    auto data = m_buffer.mid(2, m_size);
                    m_buffer.remove(0, m_size + 2);
                    m_size = 0;
                    readAgain = m_buffer.size() != 0;
                    QDns::Base::QDnsPacket packet(data);
                    if (packet.isValid())
                    {
                        emit onRequest(QDnsServer::OnRequestParams
                                               {
                                                       packet,
                                                       m_socket->peerAddress(),
                                                       m_socket->peerPort(),
                                                       [this](QDns::Server::QDnsServer::SendAnswerParams answerParams)
                                                       {
                                                           sendAnswer(answerParams);
                                                       }
                                               });
                    }
                }
            } while (readAgain);
        }
    }
}